@@ -1,10 +1,11 @@
 import logging
 import urllib
 import urllib2
+import uuid
 
 import django
 from django.conf import settings
-from django.utils.encoding import smart_unicode
+from django.utils.encoding import force_unicode
 from django.utils.hashcompat import md5_constructor
 
 from sentry import conf
@@ -61,24 +62,31 @@ def transform(value):
     # TODO: make this extendable
     # TODO: include some sane defaults, like UUID
     # TODO: dont coerce strings to unicode, leave them as strings
-    if isinstance(value, (tuple, list)):
-        return [transform(o) for o in value]
+    if isinstance(value, (tuple, list, set, frozenset)):
+        return type(value)(transform(o) for o in value)
+    elif isinstance(value, uuid.UUID):
+        return repr(value)
     elif isinstance(value, dict):
         return dict((k, transform(v)) for k, v in value.iteritems())
-    elif isinstance(value, basestring):
-        value = force_unicode(value)
+    elif isinstance(value, unicode):
+        return to_unicode(value)
+    elif isinstance(value, str):
+        try:
+            return str(value)
+        except:
+            return to_unicode(value)
     elif not isinstance(value, (int, bool)) and value is not None:
         # XXX: we could do transform(repr(value)) here
-        return force_unicode(value)
+        return to_unicode(value)
     return value
 
-def force_unicode(value):
+def to_unicode(value):
     try:
-        value = smart_unicode(value)
+        value = force_unicode(value)
     except (UnicodeEncodeError, UnicodeDecodeError):
         value = '(Error decoding value)'
     except Exception: # in some cases we get a different exception
-        value = smart_unicode(type(value))
+        value = force_unicode(type(value))
     return value
 
 def get_installed_apps():
@@ -160,4 +168,4 @@ def urlread(url, get={}, post={}, headers={}, timeout=None):
         response = urllib2.urlopen(req, urllib.urlencode(post), timeout).read()
     except:
         response = urllib2.urlopen(req, urllib.urlencode(post)).read()
-    return response
\ No newline at end of file
+    return response
