@@ -19,7 +19,6 @@ from django.core.management import call_command
 from django.core.urlresolvers import reverse
 from django.core.signals import got_request_exception
 from django.core.servers import basehttp
-from django.test.client import Client
 from django.test import TestCase
 from django.template import TemplateSyntaxError
 from django.utils.encoding import smart_unicode
@@ -29,7 +28,7 @@ from sentry.helpers import transform
 from sentry.models import Message, GroupedMessage
 from sentry.client.base import SentryClient
 from sentry.client.handlers import SentryHandler
-from sentry.client.models import sentry_exception_handler, get_client
+from sentry.client.models import get_client
 
 from models import TestModel, DuplicateKeyModel
 
@@ -590,6 +589,24 @@ class SentryTestCase(TestCase):
         
         self.assertEquals(Message.objects.count(), 3)
         self.assertEquals(GroupedMessage.objects.count(), 1)
+    
+    def testUUID(self):
+        import uuid
+        
+        logger = logging.getLogger()
+        
+        self.setUpHandler()
+        
+        uuid = uuid.uuid4()
+
+        logger.error('Test', extra={'data': {'uuid': uuid}})
+        self.assertEquals(Message.objects.count(), 1)
+        self.assertEquals(GroupedMessage.objects.count(), 1)
+        last = Message.objects.get()
+        self.assertEquals(last.logger, 'root')
+        self.assertEquals(last.level, logging.ERROR)
+        self.assertEquals(last.message, 'Test')
+        self.assertEquals(last.data['uuid'], repr(uuid))
 
 class SentryViewsTest(TestCase):
     urls = 'sentry.tests.urls'
@@ -810,7 +827,7 @@ class SentryMailTest(TestCase):
         out = mail.outbox[0]
 
         self.assertTrue('Traceback (most recent call last):' in out.body)
-        self.assertTrue("COOKIES:{'commenter_name': u'admin'," in out.body, out.body)
+        self.assertTrue("COOKIES:{'commenter_name': 'admin'," in out.body, out.body)
         self.assertEquals(out.subject, 'Error (EXTERNAL IP): /group/1')
 
     def test_mail_on_creation(self):
@@ -884,6 +901,18 @@ class SentryHelpersTest(TestCase):
         settings.DATABASES = _databases
         settings.DATABASE_ENGINE = _engine
 
+    def test_transform_handles_gettext_lazy(self):
+        from sentry.helpers import transform
+        from django.utils.functional import lazy
+
+        def fake_gettext(to_translate):
+            return u'Igpay Atinlay'
+        fake_gettext_lazy = lazy(fake_gettext, str)
+        self.assertEquals(
+            pickle.loads(pickle.dumps(
+                    transform(fake_gettext_lazy("something")))),
+            u'Igpay Atinlay')
+
 class SentryClientTest(TestCase):
     urls = 'sentry.tests.urls'
 
@@ -954,4 +983,4 @@ class SentryManageTest(TestCase):
         command = Command()
         command.handle(days=1)
         
-        self.assertEquals(Message.objects.count(), 0)
\ No newline at end of file
+        self.assertEquals(Message.objects.count(), 0)
